from django.db import models
from django.urls import reverse
from netbox.models import NetBoxModel


class Sitemap(NetBoxModel):
    name = models.CharField(
        max_length=100
    )
    site_groups = models.ManyToManyField(
        to='dcim.SiteGroup',
        related_name='sitemaps',
        blank=True
    )
    sites = models.ManyToManyField(
        to='dcim.Site',
        related_name='sitemaps',
        blank=True
    )
    comments = models.TextField(
        blank=True
    )

    class Meta:
        ordering = ("name",)

    def __str__(self):
        return self.name

    def get_absolute_url(self):
        return reverse("plugins:netbox_sitemap:sitemap", args=[self.pk])



"""Top-level package for NetBox Sitemap Plugin."""

__author__ = """Lars Däschner"""
__email__ = "contact@ldaeschner.de"
__version__ = "0.2.0"


from netbox.plugins import PluginConfig


class SitemapConfig(PluginConfig):
    name = "netbox_sitemap"
    verbose_name = "NetBox Sitemap Plugin"
    description = "NetBox plugin for displaying sites on a map."
    version = "version"
    base_url = "netbox-sitemap"


config = SitemapConfig



from netbox.views import generic
from . import filtersets, forms, models, tables


class SitemapView(generic.ObjectView):
    queryset = models.Sitemap.objects.all()


class SitemapListView(generic.ObjectListView):
    queryset = models.Sitemap.objects.all()
    table = tables.SitemapTable
    filterset = filtersets.SitemapFilterSet
    filterset_form = forms.SitemapFilterForm


class SitemapEditView(generic.ObjectEditView):
    queryset = models.Sitemap.objects.all()
    form = forms.SitemapForm


class SitemapDeleteView(generic.ObjectDeleteView):
    queryset = models.Sitemap.objects.all()



from rest_framework import serializers

from netbox.api.serializers import NetBoxModelSerializer
from ..models import Sitemap

class SitemapSerializer(NetBoxModelSerializer):
    url = serializers.HyperlinkedIdentityField(
        view_name='plugins-api:netbox_sitemap-api:sitemap-detail'
    )

    class Meta:
        model = Sitemap
        fields = (
            'id', 'url', 'display', 'name', 'site_groups', 'sites', 'comments', 'tags', 'custom_fields', 'created',
            'last_updated'
        )
        brief_fields =(
            'id', 'url', 'display', 'name', 'site_groups', 'sites'
        )


from django import forms
from netbox.forms import NetBoxModelForm, NetBoxModelFilterSetForm
from utilities.forms.fields import CommentField, DynamicModelMultipleChoiceField
from utilities.forms.rendering import FieldSet
from dcim.models import SiteGroup, Site

from .models import Sitemap


class SitemapForm(NetBoxModelForm):
    site_groups = DynamicModelMultipleChoiceField(
        label=('Site Groups'),
        queryset=SiteGroup.objects.all(),
        required=False,
        quick_add=True
    )
    sites = DynamicModelMultipleChoiceField(
        label=('Sites'),
        queryset=Site.objects.all(),
        required=False,
        quick_add=True
    )
    comments = CommentField()

    fieldsets = (
        FieldSet(
            'name', 'site_groups', 'sites', 'tags', name=('SiteMap')),
    )

    class Meta:
        model = Sitemap
        fields = ('name', 'site_groups', 'sites', 'tags', 'comments')

class SitemapFilterForm(NetBoxModelFilterSetForm):
    model = Sitemap

    site_groups = forms.ModelMultipleChoiceField(
        label=('Site Groups'),
        queryset=SiteGroup.objects.all(),
        required=False
    )
    sites = forms.ModelMultipleChoiceField(
        label=('Sites'),
        queryset=Site.objects.all(),
        required=False
    )


from netbox.api.viewsets import NetBoxModelViewSet

from .. import filtersets, models
from .serializers import SitemapSerializer

class SitemapViewSet(NetBoxModelViewSet):
    queryset = models.Sitemap.objects.prefetch_related('site_groups', 'sites', 'tags')
    serializer_class = SitemapSerializer
    filterset_class = filtersets.SitemapFilterSet



from django.urls import path
from netbox.views.generic import ObjectChangeLogView

from . import models, views


urlpatterns = (
    path("sitemaps/", views.SitemapListView.as_view(), name="sitemap_list"),
    path("sitemaps/add/", views.SitemapEditView.as_view(), name="sitemap_add"),
    path("sitemaps/<int:pk>/", views.SitemapView.as_view(), name="sitemap"),
    path("sitemaps/<int:pk>/edit/", views.SitemapEditView.as_view(), name="sitemap_edit"),
    path("sitemaps/<int:pk>/delete/", views.SitemapDeleteView.as_view(), name="sitemap_delete"),
    path(
        "sitemaps/<int:pk>/changelog/",
        ObjectChangeLogView.as_view(),
        name="sitemap_changelog",
        kwargs={"model": models.Sitemap},
    ),
)



# Generated by Django 5.2.5 on 2025-09-02 09:25

import netbox.models.deletion
import taggit.managers
import utilities.json
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('dcim', '0210_macaddress_ordering'),
        ('extras', '0129_fix_script_paths'),
    ]

    operations = [
        migrations.CreateModel(
            name='Sitemap',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False)),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                ('custom_field_data', models.JSONField(blank=True, default=dict, encoder=utilities.json.CustomFieldJSONEncoder)),
                ('name', models.CharField(max_length=100)),
                ('comments', models.TextField(blank=True)),
                ('site_groups', models.ManyToManyField(blank=True, related_name='sitemaps', to='dcim.sitegroup')),
                ('sites', models.ManyToManyField(blank=True, related_name='sitemaps', to='dcim.site')),
                ('tags', taggit.managers.TaggableManager(through='extras.TaggedItem', to='extras.Tag')),
            ],
            options={
                'ordering': ('name',),
            },
            bases=(netbox.models.deletion.DeleteMixin, models.Model),
        ),
    ]



from netbox.plugins import PluginMenuButton, PluginMenuItem


plugin_buttons = [
    PluginMenuButton(
        link="plugins:netbox_sitemap:sitemap_add",
        title="Add",
        icon_class="mdi mdi-plus-thick",
    )
]

menu_items = (
    PluginMenuItem(
        link="plugins:netbox_sitemap:sitemap_list",
        link_text="Sitemap",
        buttons=plugin_buttons,
    ),
)



import django_tables2 as tables
from netbox.tables import NetBoxTable, columns

from .models import Sitemap


class SitemapTable(NetBoxTable):
    name = tables.Column(
        linkify=True
    )
    site_groups = columns.ManyToManyColumn(
        linkify_item=True,
        verbose_name=('Site Groups')
    )
    sites = columns.ManyToManyColumn(
        linkify_item=True,
        verbose_name=('Sites')
    )

    class Meta(NetBoxTable.Meta):
        model = Sitemap
        fields = ("pk", "id", "name", "site_groups", "sites", "tags", "comments", "actions")
        default_columns = ("name", "site_groups", "sites")



from netbox.filtersets import NetBoxModelFilterSet
from .models import Sitemap


class SitemapFilterSet(NetBoxModelFilterSet):
    class Meta:
        model = Sitemap
        fields = ('id', 'name', 'site_groups', 'sites')
    def search(self, queryset, name, value):
        return queryset.filter(name__icontains=value)